import numpy as np
import binascii
import struct


class Tag2NumPy(object):

    def __init__(self, shape):
        self._input_tag_file_path = None
        self._output_dir = '.'
        self._output_numpy_array = None
        self._shape = shape

    def set_input_tag_file_path(self, file_path):
        self._input_tag_file_path = file_path

    def get_output_numpy_array(self):
        return self._output_numpy_array

    @staticmethod
    def _get_pixels(tag_file_path):
        f = open(tag_file_path, 'rb')
        f.seek(0)
        byte = f.read(1)
        # Make sure to check the byte-value in Python 3!!
        while byte != b'':
            byte_hex = binascii.hexlify(byte)
            if byte_hex == b'0c':
                break
            byte = f.read(1)
        values = []
        f.read(1)
        while byte != b'':
            v = struct.unpack('b', byte)
            values.append(v)
            byte = f.read(1)
        values = np.asarray(values)
        values = values.astype(np.uint16)
        return values

    def execute(self):
        self._output_numpy_array = self._get_pixels(self._input_tag_file_path)
        try:
            if self._shape:
                self._output_numpy_array = self._output_numpy_array.reshape(self._shape)
        except ValueError:
            self._output_numpy_array = None


if __name__ == '__main__':
    tag2numpy = Tag2NumPy((512, 512))
    tag2numpy.set_input_tag_file_path('/Volumes/USB_SECURE1/data/radiomics/projects/deepseg/data/mega/lung_len/org/LUNG1-001_20180209_CT.tag')
    tag2numpy.execute()
    print(tag2numpy.get_output_numpy_array())



import os
import shutil
import pydicom
import numpy as np
import matplotlib.pyplot as plt

from barbell2light.dicom import is_dicom_file, is_tag_file, get_tag_file_for_dicom, tag2numpy, decompress


class Tag2Dcm:

    def __init__(self):
        self.dcm_file = None
        self.tag_file = None
        self.output_dir = '.'
        self.output_dcm_file = None
        self.output_tag_file = None
        self.output_tag_dcm_file = None
        self.output_dcm_png_file = None
        self.output_tag_dcm_png_file = None
        self.copy_original_dcm_file_to_output_dir = False
        self.copy_original_tag_file_to_output_dir = False
        self.png_figure_size = (10, 10)
        self.create_pngs = False
        self.verbose = False

    def set_dicom_and_tag_file(self, dcm_file, tag_file):
        if not is_dicom_file(dcm_file):
            raise RuntimeError(f'File {dcm_file} is not a DICOM file')
        if not is_tag_file(tag_file):
            raise RuntimeError(f'File {tag_file} does not have .tag extension')
        if get_tag_file_for_dicom(dcm_file) != tag_file:
            raise RuntimeError(f'Files {dcm_file} and {tag_file} do not seem to belong together')
        self.dcm_file = dcm_file
        self.tag_file = tag_file

    def set_output_dir(self, output_dir):
        if not os.path.isdir(output_dir):
            print(f'Output directory {output_dir} does not exist, creating it...')
            os.makedirs(output_dir, exist_ok=False)
        self.output_dir = output_dir

    def get_output_dir(self):
        return self.output_dir

    def set_copy_original_dcm_file_to_output_dir(self, copy_original_dcm_file_to_output_dir):
        self.copy_original_dcm_file_to_output_dir = copy_original_dcm_file_to_output_dir

    def set_copy_original_tag_file_to_output_dir(self, copy_original_tag_file_to_output_dir):
        self.copy_original_tag_file_to_output_dir = copy_original_tag_file_to_output_dir

    def set_png_figure_size(self, png_figure_size):
        self.png_figure_size = png_figure_size

    def get_png_figure_size(self):
        return self.png_figure_size

    def set_create_pngs(self, create_pngs):
        self.create_pngs = create_pngs

    def set_verbose(self, verbose):
        self.verbose = verbose

    @staticmethod
    def apply_ct_window(pix, window):
        result = (pix - window[1] + 0.5 * window[0])/window[0]
        result[result < 0] = 0
        result[result > 1] = 1
        return result

    @staticmethod
    def get_color_map():
        color_map = []
        for i in range(256):
            if i == 1:  # muscle
                color_map.append([255, 0, 0])
            elif i == 2:  # inter-muscular adipose tissue
                color_map.append([0, 255, 0])
            elif i == 5:  # visceral adipose tissue
                color_map.append([255, 255, 0])
            elif i == 7:  # subcutaneous adipose tissue
                color_map.append([0, 255, 255])
            elif i == 12:  # unknown
                color_map.append([0, 0, 255])
            else:
                color_map.append([0, 0, 0])
        return color_map

    def execute(self):
        p = pydicom.dcmread(self.dcm_file)
        if p.file_meta.TransferSyntaxUID.is_compressed:
            p.decompress()
        pixels = p.pixel_array
        pixels = pixels * p.RescaleSlope + p.RescaleIntercept
        pixels = self.apply_ct_window(pixels, [400, 50])
        pixels = pixels.astype(float)
        pixels_org = pixels.copy()
        converter = tag2numpy.Tag2NumPy(pixels.shape)
        converter.set_input_tag_file_path(self.tag_file)
        converter.execute()
        pixels_tag = converter.get_output_numpy_array()
        pixels_new = np.zeros((*pixels_tag.shape, 3), dtype=np.uint8)
        np.take(self.get_color_map(), pixels_tag, axis=0, out=pixels_new)
        p.PhotometricInterpretation = 'RGB'
        p.SamplesPerPixel = 3
        p.BitsAllocated = 8
        p.BitsStored = 8
        p.HighBit = 7
        p.add_new(0x00280006, 'US', 0)
        p.is_little_endian = True
        p.fix_meta_info()
        p.PixelData = pixels_new.tobytes()
        p.SOPInstanceUID = '{}.9999'.format(p.SOPInstanceUID)
        self.output_dcm_file = os.path.join(self.output_dir, os.path.split(self.dcm_file)[1])
        if self.copy_original_dcm_file_to_output_dir:
            shutil.copy(self.dcm_file, self.output_dir)
        self.output_tag_file = os.path.join(self.output_dir, os.path.split(self.tag_file)[1])
        if self.copy_original_tag_file_to_output_dir:
            shutil.copy(self.tag_file, self.output_dir)
        self.output_tag_dcm_file = os.path.join(self.output_dir, os.path.split(self.tag_file)[1] + '.dcm')
        p.save_as(self.output_tag_dcm_file)
        if self.create_pngs:
            fig = plt.figure(figsize=self.png_figure_size)
            ax = fig.add_subplot(1, 1, 1)
            plt.imshow(pixels_org, cmap='gray')
            ax.axis('off')
            self.output_dcm_png_file = os.path.join(self.output_dir, self.get_output_dcm_file_name() + '.png')
            plt.savefig(self.output_dcm_png_file, bbox_inches='tight')
            fig = plt.figure(figsize=self.png_figure_size)
            ax = fig.add_subplot(1, 1, 1)
            plt.imshow(pixels_new)
            ax.axis('off')
            self.output_tag_dcm_png_file = os.path.join(self.output_dir, self.get_output_tag_dcm_file_name() + '.png')
            plt.savefig(self.output_tag_dcm_png_file, bbox_inches='tight')
            plt.close('all')

    def get_output_dcm_file(self):
        return self.output_dcm_file

    def get_output_dcm_file_name(self):
        return os.path.split(self.get_output_dcm_file())[1]

    def get_output_dcm_png_file(self):
        return self.output_dcm_png_file

    def get_output_dcm_png_file_name(self):
        return os.path.split(self.get_output_dcm_png_file())[1]

    def get_output_tag_file(self):
        return self.output_tag_file

    def get_output_tag_file_name(self):
        return os.path.split(self.get_output_tag_file())[1]

    def get_output_tag_dcm_file(self):
        return self.output_tag_dcm_file

    def get_output_tag_dcm_file_name(self):
        return os.path.split(self.get_output_tag_dcm_file())[1]

    def get_output_tag_dcm_png_file(self):
        return self.output_tag_dcm_png_file

    def get_output_tag_dcm_png_file_name(self):
        return os.path.split(self.get_output_tag_dcm_png_file())[1]


if __name__ == '__main__':
    t2d = Tag2Dcm()
    t2d.set_dicom_and_tag_file(
        dcm_file='../../data/10.dcm',
        tag_file='../../data/10.tag',
    )
    t2d.set_output_dir(output_dir='../../data')
    t2d.set_copy_original_dcm_file_to_output_dir(False)
    t2d.set_copy_original_tag_file_to_output_dir(False)
    t2d.execute()


# import os
# import numpy as np
# import binascii
# import struct
# import SimpleITK as sitk
#
#
# class Tag2Dcm(object):
#
#     def __init__(self):
#         self._input_dicom_file_path = None
#         self._input_tag_file_path = None
#         self._output_dir = '.'
#         self._output_file_path = None
#         self._overwrite_output = False
#         self._shape = None
#         self._spacing = None
#         self._origin = None
#         self._direction = None
#         self._verbose = True
#
#     # INTERFACE
#
#     def set_input_dicom_file_path(self, file_path):
#         self._input_dicom_file_path = file_path
#
#     def set_input_tag_file_path(self, file_path):
#         self._input_tag_file_path = file_path
#
#     def set_output_dir(self, output_dir):
#         self._output_dir = output_dir
#
#     def get_output_file_path(self):
#         return self._output_file_path
#
#     def set_overwrite_output(self, value):
#         self._overwrite_output = value
#
#     def set_verbose(self, verbose):
#         self._verbose = verbose
#
#     # INTERNAL METHODS
#
#     def _get_info_from_dicom(self, f, verbose):
#         reader = sitk.ImageFileReader()
#         reader.SetImageIO('GDCMImageIO')
#         reader.SetFileName(f)
#         image = reader.Execute()
#         self._shape = image.GetSize()
#         self._spacing = image.GetSpacing()
#         self._origin = image.GetOrigin()
#         self._direction = image.GetDirection()
#         min_max_filter = sitk.MinimumMaximumImageFilter()
#         min_max_filter.Execute(image)
#         minimum = min_max_filter.GetMinimum()
#         maximum = min_max_filter.GetMaximum()
#         # p = pydicom.read_file(f)
#         # # Make sure to put the 1 at the front because the NumPy indexing is differently ordered than
#         # # SimpleITK pixel indexing
#         # self._shape = (1, p.Rows, p.Columns)
#         # self._spacing = (float(p.PixelSpacing[0]), float(p.PixelSpacing[1]), 1.0)
#         # self._origin = (
#         #     float(p.ImagePositionPatient[0]),
#         #     float(p.ImagePositionPatient[1]),
#         #     float(p.ImagePositionPatient[2])
#         # )
#         # self._direction = (
#         #     float(p.ImageOrientationPatient[0]),
#         #     float(p.ImageOrientationPatient[1]),
#         #     float(p.ImageOrientationPatient[2]),
#         #     float(p.ImageOrientationPatient[3]),
#         #     float(p.ImageOrientationPatient[4]),
#         #     float(p.ImageOrientationPatient[5]),
#         #     0, 0, 1,
#         # )
#         if verbose:
#             print('File: {}, Size: {}, Spacing: {}, Origin: {}, Direction: {}, Min: {}, Max: {}'.format(
#                 f, self._shape, self._spacing, self._origin, self._direction, minimum, maximum))
#
#     @staticmethod
#     def _get_pixels(tag_file_path):
#         f = open(tag_file_path, 'rb')
#         f.seek(0)
#         byte = f.read(1)
#         # Make sure to check the byte-value in Python 3!!
#         while byte != b'':
#             byte_hex = binascii.hexlify(byte)
#             if byte_hex == b'0c':
#                 break
#             byte = f.read(1)
#         values = []
#         f.read(1)
#         while byte != b'':
#             v = struct.unpack('b', byte)
#             values.append(v)
#             byte = f.read(1)
#         values = np.asarray(values)
#         values = values.astype(np.uint16)
#         return values
#
#     # EXECUTE
#
#     def execute(self):
#         os.makedirs(self._output_dir, exist_ok=True)
#         self._get_info_from_dicom(self._input_dicom_file_path, self._verbose)
#         pixels = self._get_pixels(self._input_tag_file_path)
#
#         # pixels.shape = self._shape
#         pixels.shape = (1, self._shape[1], self._shape[0])
#         image = sitk.GetImageFromArray(pixels)
#         image.SetSpacing(self._spacing)
#         image.SetOrigin(self._origin)
#         image.SetDirection(self._direction)
#         # print(image)
#
#         # f = open(self._input_dicom_file_path, 'rb')
#         # p = pydicom.read_file(f)
#         # p.pixel_array.setflags(write=True)
#         # p.pixel_array.flat = pixels
#         # p.PixelData = p.pixel_array.tobytes()
#         # p.RescaleIntercept = 0
#         # p.RescaleSlope = 1
#
#         file_name = os.path.split(self._input_tag_file_path)
#         file_name = file_name[1]
#         file_name = os.path.splitext(file_name)
#         file_name = file_name[0]
#         file_name = '{}_tag.dcm'.format(file_name)
#         file_path = os.path.join(self._output_dir, file_name)
#
#         try:
#             os.stat(file_path)
#             if not self._overwrite_output:
#                 print('Error: file {} already exists!'.format(file_path))
#                 return
#         except FileNotFoundError:
#             pass
#
#         self._output_file_path = file_path
#
#         writer = sitk.ImageFileWriter()
#         writer.SetFileName(self._output_file_path)
#         writer.SetImageIO('GDCMImageIO')
#         writer.Execute(image)
#         # p.save_as(file_path)
#         # f.close()
#         # print(self._output_file_path)
#
#
# if __name__ == '__main__':
#     node = Tag2Dcm()
#     node.set_input_dicom_file_path('/Volumes/USB_SECURE1/data/radiomics/projects/004_ovarium/data/test/IM_1.dcm')
#     node.set_input_tag_file_path('/Volumes/USB_SECURE1/data/radiomics/projects/004_ovarium/data/test/IM_1.tag')
#     node.set_output_dir('/Volumes/USB_SECURE1/data/radiomics/projects/004_ovarium/data/test/out')
#     node.set_overwrite_output(True)
#     node.execute()



import os
import json

from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session
from barbell2light.utils import Logger


class CastorClient:

    def __init__(self, client_id=None, client_secret=None, log_dir='.', cache_dir='castor_cache'):
        client_id, client_secret = self.get_credentials(client_id, client_secret)
        self.base_url = 'https://data.castoredc.com'
        self.token_url = self.base_url + '/oauth/token'
        self.api_url = self.base_url + '/api'
        self.session = self.create_session(client_id, client_secret, self.token_url)
        self.logger = Logger(prefix='log_castorclient', to_dir=log_dir)
        self.cache_dir = cache_dir

    @staticmethod
    def get_credentials(client_id=None, client_secret=None):
        cid = client_id
        cse = client_secret
        if cid is None:
            cid_file = '{}/castorclientid.txt'.format(os.environ['HOME'])
            if os.path.isfile(cid_file):
                with open(cid_file, 'r') as f:
                    cid = f.readline().strip()
            else:
                cid = os.environ.get('CASTOR_CLIENT_ID', None)
                if cid is None:
                    raise RuntimeError('Castor client ID not found in: \n'
                                       '(1) arguments\n'
                                       '(2) $HOME/castorclientid.txt\n'
                                       '(3) CASTOR_CLIENT_ID environment variable')
        if cse is None:
            cse_file = '{}/castorclientsecret.txt'.format(os.environ['HOME'])
            if os.path.isfile(cse_file):
                with open(cse_file, 'r') as f:
                    cse = f.readline().strip()
            else:
                cse = os.environ.get('CASTOR_CLIENT_SECRET', None)
                if cse is None:
                    raise RuntimeError('Castor client secret not found in: \n'
                                       '(1) arguments\n'
                                       '(2) $HOME/castorclientsecret.txt\n'
                                       '(3) CASTOR_CLIENT_SECRET environment variable')
        return cid, cse


    @staticmethod
    def create_session(client_id, client_secret, token_url):
        client = BackendApplicationClient(client_id=client_id)
        client_session = OAuth2Session(client=client)
        client_session.fetch_token(
            token_url=token_url,
            client_id=client_id,
            client_secret=client_secret,
        )
        return client_session

    def get_studies(self):
        response = self.session.get(self.api_url + '/study').json()
        studies = []
        for study in response['_embedded']['study']:
            studies.append(study)
        return studies

    def get_study_id(self, name):
        studies = self.get_studies()
        for study in studies:
            if study['name'] == name:
                return study['study_id']
        return None

    def get_fields(self, study_id, use_cache=True, verbose=False):
        if use_cache and os.path.isfile('{}/fields_{}.json'.format(self.cache_dir, study_id)):
            self.logger.print('Loading fields from {}/fields_{}'.format(self.cache_dir, study_id))
            fields = json.load(open('{}/fields_{}.json'.format(self.cache_dir, study_id), 'r'))
            return fields
        self.logger.print('Loading fields for study {} from Castor'.format(study_id))
        url = self.api_url + '/study/{}/field'.format(study_id)
        response = self.session.get(url).json()
        page_count = response['page_count']
        fields = []
        for i in range(1, page_count + 1):
            url = self.api_url + '/study/{}/field?page={}'.format(study_id, i)
            response = self.session.get(url).json()
            for field in response['_embedded']['fields']:
                fields.append(field)
                if verbose:
                    self.logger.print(field['id'])
        if use_cache:
            os.makedirs('{}'.format(self.cache_dir), exist_ok=True)
            json.dump(fields, open('{}/fields_{}.json'.format(self.cache_dir, study_id), 'w'))
        return fields

    @staticmethod
    def get_field(name, fields):
        for f in fields:
            if f['field_variable_name'] == name:
                return f
        return None

    def get_field_id(self, name, fields):
        f = self.get_field(name, fields)
        return f['id']

    @staticmethod
    def get_option_name(value, option_group_name, option_groups):
        for option_group in option_groups:
            if option_group['name'] == option_group_name:
                for option in option_group['options']:
                    if option['value'] == value:
                        return option['name']
        return None

    def get_records(self, study_id, use_cache=True, verbose=False):
        if use_cache and os.path.isfile('{}/records_{}.json'.format(self.cache_dir, study_id)):
            self.logger.print('Loading records from {}/records_{}.json'.format(self.cache_dir, study_id))
            records = json.load(open('{}/records_{}.json'.format(self.cache_dir, study_id), 'r'))
            return records
        self.logger.print('Loading records for study {} from Castor'.format(study_id))
        url = self.api_url + '/study/{}/record'.format(study_id)
        response = self.session.get(url).json()
        page_count = response['page_count']
        records = []
        for i in range(1, page_count + 1):
            url = self.api_url + '/study/{}/record?page={}'.format(study_id, i)
            response = self.session.get(url).json()
            for record in response['_embedded']['records']:
                if not record['id'].startswith('ARCHIVED'):
                    records.append(record)
                    if verbose:
                        self.logger.print(record['id'])
        if use_cache:
            os.makedirs('{}'.format(self.cache_dir), exist_ok=True)
            json.dump(records, open('{}/records_{}.json'.format(self.cache_dir, study_id), 'w'))
        return records

    def get_option_groups(self, study_id, verbose=False):
        url = self.api_url + '/study/{}/field-optiongroup'.format(study_id)
        response = self.session.get(url).json()
        page_count = response['page_count']
        option_groups = []
        for i in range(1, page_count + 1):
            url = self.api_url + '/study/{}/field-optiongroup?page={}'.format(study_id, i)
            response = self.session.get(url).json()
            for option_group in response['_embedded']['fieldOptionGroups']:
                option_groups.append(option_group)
                if verbose:
                    self.logger.print(option_group['id'])
        return option_groups

    def get_field_data(self, study_id, record_id, field_id):
        url = self.api_url + '/study/{}/record/{}/study-data-point/{}'.format(study_id, record_id, field_id)
        field_data = self.session.get(url).json()
        return field_data


if __name__ == '__main__':
    client = CastorClient()
    print(client.get_studies())



import os
import numpy as np

from pydicom._dicom_dict import DicomDictionary
from .tag2numpy import Tag2NumPy


def is_dicom_file(file_path_or_obj):
    file_obj = file_path_or_obj
    if isinstance(file_obj, str):
        if not os.path.isfile(file_obj):
            return False
        if file_obj.startswith('._'):
            return False
        file_obj = open(file_obj, "rb")
    try:
        result = file_obj.read(132).decode('ASCII')[-4:] == 'DICM'
        file_obj.seek(0)
        return result
    except UnicodeDecodeError:
        return False


def is_tag_file(file_path):
    return file_path.endswith('.tag') and not file_path.startswith('._')


def get_tag_file_for_dicom(dcm_file):
    tag_file = os.path.splitext(dcm_file)[0] + '.tag'
    if not os.path.isfile(tag_file):
        tag_file = dcm_file + '.tag'
        if not os.path.isfile(tag_file):
            print(f'Could not find TAG file for DICOM file {dcm_file}')
            return None
        return tag_file
    return tag_file


def get_dicom_tag_for_name(name):
    for key, value in DicomDictionary.items():
        if name == value[4]:
            return hex(int(key))
    return None


def get_dictionary_items():
    return DicomDictionary.items()


def get_pixels(p, normalize=False):
    pixels = p.pixel_array
    if not normalize:
        return pixels
    if normalize is True:
        return p.RescaleSlope * pixels + p.RescaleIntercept
    if isinstance(normalize, int):
        return (pixels + np.min(pixels)) / (np.max(pixels) - np.min(pixels)) * normalize
    if isinstance(normalize, list):
        return (pixels + np.min(pixels)) / (np.max(pixels) - np.min(pixels)) * normalize[1] + normalize[0]
    return pixels


def get_tag_pixels(f, shape=None):
    converter = Tag2NumPy(shape)
    converter.set_input_tag_file_path(f)
    converter.execute()
    return converter.get_output_numpy_array()


def is_compressed(p):
    try:
        p.convert_pixel_data()
        return False
    except RuntimeError:
        return True


def decompress(f, reuse_name=False):
    result = os.system('which gdcmconv>/dev/null')
    if result > 0:
        raise RuntimeError('Tool "gdcmconv" is not installed')
    items = os.path.splitext(f)
    base_name, extension = items[0], items[1]
    f_target = base_name + '_raw' + extension
    command = 'gdcmconv --raw {} {}'.format(f, f_target)
    os.system(command)
    return f_target



import os
import cmd2
import pydicom

from barbell2light.dicom import is_dicom_file, get_dicom_tag_for_name, get_dictionary_items


class DicomExplorer:

    def __init__(self):
        self.files = []

    # LOAD

    def load_file(self, f, verbose=True):
        if not os.path.isfile(f):
            if verbose:
                print('Cannot find file {}'.format(f))
            return
        if is_dicom_file(f):
            self.files.append(f)
            if verbose:
                print(f)

    def load_dir(self, d, verbose=True):
        if not os.path.isdir(d):
            if verbose:
                print('Cannot find directory {}'.format(d))
            return
        for root, dirs, files in os.walk(d):
            for f in files:
                f = os.path.join(root, f)
                self.load_file(f, verbose)
        if verbose:
            print('Loaded {} files'.format(len(self.files)))

    # CONVERT

    def to_raw(self, d_out, verbose=True):
        result = os.system('which gdcmconv>/dev/null')
        if result > 0:
            raise RuntimeError('Tool "gdcmconv" is not installed')
        os.makedirs(d_out, exist_ok=False)
        for f in self.files:
            f_target = os.path.join(d_out, os.path.split(f)[1])
            command = 'gdcmconv --raw {} {}'.format(f, f_target)
            os.system(command)
            if verbose:
                print('Converted {}'.format(f))

    # INSPECTION

    @staticmethod
    def get_header(f, verbose=True):
        if os.path.isfile(f) and is_dicom_file(f):
            p = pydicom.read_file(f, stop_before_pixels=True)
            if verbose:
                print(p)
            return p
        return None

    @staticmethod
    def get_pixel_data(f):
        if os.path.isfile(f) and is_dicom_file(f):
            p = pydicom.read_file(f, stop_before_pixels=False)
            return p.pixel_array
        return None


    @staticmethod
    def get_tags(key_word='', verbose=True):
        outputs = []
        for key, value in get_dictionary_items():
            output = '{}: {}'.format(key, value)
            if key_word == '':
                outputs.append(output)
                if verbose:
                    print(output)
            else:
                for item in value:
                    if key_word in item:
                        outputs.append(output)
                        if verbose:
                            print(output)
        return outputs

    def get_tag_values(self, tag_name, verbose=True):
        tag = get_dicom_tag_for_name(tag_name)
        if verbose:
            print(tag)
        values = {}
        for f in self.files:
            p = pydicom.read_file(f)
            if tag in list(p.keys()):
                values[f] = p[tag].value
                if verbose:
                    print('{}: {}'.format(f, values[f]))
        return values

    def check_pixels(self, verbose=True):
        bad_files = []
        for f in self.files:
            p = pydicom.read_file(f)
            try:
                p.convert_pixel_data()
            except NotImplementedError:
                bad_files.append(f)
                if verbose:
                    print('ERROR: {}'.format(f))
        return bad_files


class DicomExplorerShell(cmd2.Cmd):

    def __init__(self):
        super(DicomExplorerShell, self).__init__()
        self.intro = 'Welcome to the DICOM Explorer Shell!'
        self.prompt = '(dicom) '
        self.debug = True
        self.explorer = DicomExplorer()

    def do_load_file(self, f):
        """ Usage: load_file <file>
        Loads single DICOM file <file>
        """
        self.explorer.load_file(f)
        self.poutput('Done')

    def do_load_dir(self, d):
        """ Usage: load_dir <directory>
        Loads all DICOM files (recursively) in given <directory>"""
        self.explorer.load_dir(d)
        self.poutput('Done')

    def do_to_raw(self, d_out='.'):
        """ Usage: to_raw
        Converts all loaded DICOM files to RAW format using GDCM"""
        self.explorer.to_raw(d_out)
        self.poutput('Done')

    def do_show_header(self, f):
        """ Usage: show_header <file>
        Show header information for DICOM file <file>"""
        self.explorer.get_header(f)
        self.poutput('Done')

    def do_show_tags(self, key_word):
        """ Usage: show_tags <key_word>
        Show all tags containing <key_word>"""
        self.explorer.get_tags(key_word)
        self.poutput('Done')

    def do_show_tag_values(self, tag_name):
        """ Usage: show_tag_values <tag_name>
        For all loaded DICOM files, show value of tag <tag_name>"""
        tag_values = self.explorer.get_tag_values(tag_name)
        with open('tag_values.txt', 'w') as f:
            for k in tag_values.keys():
                f.write('{}: {}\n'.format(k, tag_values[k]))
        self.poutput('Done (written string output to tag_values.txt)')

    def do_check_pixels(self, _):
        """ Usage: check_pixels
        For all loaded DICOM files, check whether the pixels can be loaded into a NumPy array. If not,
        the pixel values probably need to be converted to RAW format. Use the to_raw() function for that"""
        self.explorer.check_pixels()
        self.poutput('Done')


def main():
    import sys
    shell = DicomExplorerShell()
    sys.exit(shell.cmdloop())


if __name__ == '__main__':
    main()
